@@ -5,10 +5,6 @@ dedupe provides the main user interface for the library the
 Dedupe class
 """
 
-try:
-    import json
-except ImportError: 
-    import simplejson as json
 import itertools
 import logging
 import pickle
@@ -20,10 +16,7 @@ import copy
 import os
 from collections import defaultdict
 
-try:
-    from collections import OrderedDict
-except ImportError :
-    from dedupe.backport import OrderedDict
+from dedupe.backport import OrderedDict, json
 
 import dedupe
 import dedupe.sampling as sampling
@@ -139,7 +132,8 @@ class Matching(object):
         return clusters
 
     def _checkRecordType(self, record) :
-        for field, _ in self.data_model.field_comparators :
+        for field_comparator in self.data_model.field_comparators :
+            field = field_comparator[0]
             if field not in record :
                 raise ValueError("Records do not line up with data model. "
                                  "The field '%s' is in data_model but not "
@@ -609,8 +603,8 @@ class ActiveMatching(Matching) :
                            (len(self.data_model['fields']), ))]
 
         self.training_data = numpy.zeros(0, dtype=training_dtype)
-        self.training_pairs = dedupe.backport.OrderedDict({'distinct': [], 
-                                                           'match': []})
+        self.training_pairs = OrderedDict({'distinct': [], 
+                                           'match': []})
 
 
     def cleanupTraining(self) : # pragma : no cover
@@ -1096,9 +1090,8 @@ class StaticGazetteer(StaticRecordLink, GazetteerMatching):
 
 def predicateGenerator(data_model) :
     predicates = set([])
-    for definition in data_model['fields'] :
-        if hasattr(definition, 'predicates') :
-            predicates.update(definition.predicates)
+    for definition in data_model.primary_fields :
+        predicates.update(definition.predicates)
 
     return predicates
 
