@@ -1,21 +1,19 @@
 import itertools
 import dedupe
-import predicates as predicates
+import dedupe.distance
+import predicates
 import numpy
 from collections import defaultdict
 
-from dedupe.distance.affinegap import normalizedAffineGapDistance
-from dedupe.distance.haversine import compareLatLong
-from dedupe.distance.categorical import CategoricalComparator
+from affinegap import normalizedAffineGapDistance
+from haversine import haversine
+from categorical import CategoricalComparator
 
-try:
-    from collections import OrderedDict
-except ImportError :
-    from dedupe.backport import OrderedDict
+from dedupe.backport import OrderedDict
 
 class Variable(object) :
-    def __lt__(self, other) :
-        return self.sort_level < other.sort_level
+    def __len__(self) :
+        return 1
 
     def __repr__(self) :
         return self.name
@@ -40,8 +38,6 @@ class Variable(object) :
             self.has_missing = False
 
 class FieldType(Variable) :
-    sort_level = 0
-             
     def __init__(self, definition) :
         self.field = definition['field']
 
@@ -69,8 +65,20 @@ class ExactType(FieldType) :
         else :
             return numpy.nan
 
+class PriceType(FieldType) :
+    _predicate_functions = [predicates.orderOfMagnitude,
+                            predicates.wholeFieldPredicate,
+                            predicates.roundTo1]
+    type = "Price"
 
-
+    @staticmethod
+    def comparator(price_1, price_2) :
+        if price_1 == 0 :
+            return numpy.nan
+        elif price_2 == 0 :
+            return numpy.nan
+        else :
+            return abs(numpy.log10(price_1) - numpy.log10(price_2))
 
 class ShortStringType(FieldType) :
     comparator = normalizedAffineGapDistance
@@ -121,11 +129,17 @@ class TextType(StringType) :
         self.comparator = dedupe.distance.CosineTextSimilarity(definition['corpus'])
 
 class LatLongType(FieldType) :
-    comparator = compareLatLong
     type = "LatLong"
 
     _predicate_functions = [predicates.latLongGridPredicate]
 
+    @staticmethod
+    def comparator(field_1, field_2) :
+        if field_1 == (0.0,0.0) or field_2 == (0.0,0.0) :
+            return numpy.nan
+        else :
+            return haversine(field_1, field_2)
+
 class SetType(FieldType) :
     type = "Set"
 
@@ -172,35 +186,28 @@ class CategoricalType(FieldType) :
         categories = self._categories(definition)
 
         self.comparator = CategoricalComparator(categories)
+  
+        self.higher_vars = []
+        for higher_var in self.comparator.dummy_names :
+            dummy_var = DerivedType({'name' : higher_var,
+                                     'type' : 'Dummy',
+                                     'has missing' : self.has_missing})
+            self.higher_vars.append(dummy_var)
 
-        self.dummies = []
-
-        for value, combo in sorted(self.comparator.combinations[2:]) :
-            dummy_object = HigherDummyType({'combo' : combo, 
-                                            'value' : value,
-                                            'base name' : self.name,
-                                            'has missing' : self.has_missing})
-            self.dummies.append(dummy_object)
+    def __len__(self) :
+        return len(self.comparator.dummy_names)
 
 
-class ExistsType(FieldType) :
+class ExistsType(CategoricalType) :
     type = "Exists"
     _predicate_functions = [predicates.existsPredicate]
 
     def __init__(self, definition) :
 
-        super(ExistsType, self ).__init__(definition)
+        super(CategoricalType, self ).__init__(definition)
         
         self.cat_comparator = CategoricalComparator([0, 1])
 
-        self.dummies = []
-
-        for value, combo in sorted(self.cat_comparator.combinations[2:]) :
-            dummy_object = HigherDummyType({'combo' : combo, 
-                                            'value' : value,
-                                            'base name' : self.name,
-                                            'has missing' : self.has_missing})
-            self.dummies.append(dummy_object)
 
     def comparator(self, field_1, field_2) :
         if field_1 and field_2 :
@@ -210,38 +217,16 @@ class ExistsType(FieldType) :
         else :
             return self.cat_comparator(0, 0)
 
-
-
-class SourceType(CategoricalType) :
-    type = "Source"
-
-    def _categories(self, definition) :
-        try :
-            categories = definition["sources"]
-        except KeyError :
-            raise ValueError('No "sources" defined')
-
-        if len(categories) != 2 :
-            raise ValueError("You must supply two and only " 
-                             "two source names")
-        
-        return categories
-
-class HigherDummyType(Variable) :
-    sort_level = 1
-    
-    type = "HigherOrderDummy"
+class DerivedType(Variable) :
+    type = "Derived"
 
     def __init__(self, definition) :
-        self.name = "(%s: %s)" % (str(definition['combo']), self.type)
-        self.value = definition['value']
-        self.base_name = definition['base name']
+        self.name = "(%s: %s)" % (str(definition['name']), 
+                                  str(definition['type']))
+        super(DerivedType, self).__init__(definition)
 
-        super(HigherDummyType, self).__init__(definition)
 
 class InteractionType(Variable) :
-    sort_level = 2
-
     type = "Interaction"
     
     def __init__(self, definition) :
@@ -253,8 +238,6 @@ class InteractionType(Variable) :
 
         super(InteractionType, self).__init__(definition)
 
-
-
     def expandInteractions(self, field_model) :
 
         self.interaction_fields = self.atomicInteractions(self.interactions,
@@ -263,6 +246,24 @@ class InteractionType(Variable) :
             if field_model[field].has_missing :
                 self.has_missing = True
 
+        self.categorical(field_model)
+    
+    def categorical(self, field_model) :
+        categoricals = [field for field in self.interaction_fields
+                        if hasattr(field_model[field], "higher_vars")]
+        noncategoricals = [field for field in self.interaction_fields
+                           if not hasattr(field_model[field], "higher_vars")]
+
+        dummies = [field_model[field].higher_vars 
+                   for field in categoricals]
+
+        self.higher_vars = []
+        for combo in itertools.product(*dummies) :
+            var_names = [field.name for field in combo] + noncategoricals
+            higher_var = InteractionType({'has missing' : self.has_missing,
+                                          'interaction variables' : var_names})
+            self.higher_vars.append(higher_var)
+
     def atomicInteractions(self, interactions, field_model) :
         atomic_interactions = []
 
@@ -282,48 +283,12 @@ class InteractionType(Variable) :
 
         return atomic_interactions
 
-
-    def dummyInteractions(self, field_model) :
-        dummy_interactions = []
-
-        categoricals = defaultdict(list)
-
-        for field in field_model.values() :
-            if hasattr(field, 'base_name') :
-                if field.base_name in self.interaction_fields :
-                    categoricals[field.base_name].append(field.name)
-
-        for base_name in categoricals :
-            categoricals[base_name].append(base_name)
-
-        base_combination = set([tuple(categoricals.keys())])
-
-        categorical_combinations = itertools.product(*categoricals.values())
-        categorical_combinations = set(categorical_combinations)
-        categorical_combinations -= base_combination
-
-        non_categoricals = tuple(set(self.interaction_fields) 
-                                 - set(categoricals.keys()))
-
-        for level in categorical_combinations :
-            interaction_fields = level + non_categoricals
-            interaction_variable = InteractionType(
-                {"interaction variables" : interaction_fields,
-                 "has missing" : self.has_missing})
-            interaction_variable.expandInteractions(field_model)
-
-            dummy_interactions.append(interaction_variable)
-
-        return dummy_interactions
-
 class MissingDataType(Variable) :
-    sort_level = 3
-
     type = "MissingData"
 
     def __init__(self, name) :
         
-        self.name = "(%s: Not Missing)" %name
+        self.name = "(%s: Not Missing)" % name
         self.weight = 0
 
         self.has_missing = False
