@@ -12,10 +12,7 @@ import core
 import numpy
 import logging
 import random
-from zope.index.text.textindex import TextIndex
-from zope.index.text.lexicon import Lexicon
-from zope.index.text.lexicon import Splitter
-
+import index
 
 logger = logging.getLogger(__name__)
 
@@ -69,7 +66,7 @@ class ActiveLearning(object) :
 
 def semiSupervisedNonDuplicates(data_sample,
                                 data_model,
-                                nonduplicate_confidence_threshold=.7,
+                                nonduplicate_confidence_threshold=.9,
                                 sample_size=2000):
 
     confidence = 1 - nonduplicate_confidence_threshold
@@ -153,7 +150,7 @@ def blockTraining(training_pairs,
     # blocks.
     for pred in predicate_set[:] :
         blocks = coverage.blocks[pred].itervalues()
-        if any(len(block) >= 100 for block in blocks if block) :
+        if any(len(block) >= 200 for block in blocks if block) :
             predicate_set.remove(pred)
 
     # As an efficency, we can throw away the predicates that cover too
@@ -169,8 +166,8 @@ def blockTraining(training_pairs,
             predicate_set.remove(pred)
 
 
-    distinct_coverage = coverage.predicateCoverage(predicate_set, 
-                                                   training_distinct)
+    #distinct_coverage = coverage.predicateCoverage(predicate_set, 
+    #                                               training_distinct)
 
     final_predicate_set = findOptimumBlocking(training_dupes,
                                               predicate_set,
@@ -381,17 +378,21 @@ class RecordLinkCoverage(Coverage) :
         
 
 def stopWords(data) :
-    index = TextIndex(Lexicon(Splitter()))
+    tf_index = index.CanopyIndex([])
 
     for i, (_, doc) in enumerate(data, 1) :
-        index.index_doc(i, doc)
+        try :
+            tf_index.index_doc(i, doc)
+        except :
+            print doc
+            raise
 
-    doc_freq = [(len(index.index._wordinfo[wid]), word) 
-                for word, wid in index.lexicon.items()]
+    doc_freq = [(len(tf_index.index._wordinfo[wid]), word) 
+                for word, wid in tf_index.lexicon.items()]
 
     doc_freq.sort(reverse=True)
 
-    N = float(index.index.documentCount())
+    N = float(tf_index.index.documentCount())
     threshold = int(max(1000, N * 0.05))
 
     stop_words = set([])
